//
//  MembershipListDataSource.swift
//  SampleApp
//
//  Created by Federico Zanetello on 4/10/19.
//  Copyright © 2019 David Zhang. All rights reserved.
//

import AmitySDK

final class MembershipListDataSource {
    private unowned var client: AmityClient
    private let channelId: String
    private let subChannelId: String
    private let currentUserId: String

    weak var dataSourceObserver: DataSourceListener?
   
    private var channelModeration: AmityChannelModeration
    private var channelMembership: AmityChannelMembership
    private var moderationToken: AmityNotificationToken?
    private var participationToken: AmityNotificationToken?
    
    var filter: AmityChannelMembershipFilter = .all
    var sortBy: AmitySortBy = .lastCreated
    var rolesToFilter: [String] = []
    
    var memberCollection: AmityCollection<AmityChannelMember>?
    
    init(client: AmityClient, channelId: String, subChannelId: String) {
        self.client = client
        self.channelId = channelId
        self.subChannelId = subChannelId
        self.currentUserId = client.currentUserId ?? ""
        
        channelModeration = AmityChannelModeration(client: client, andChannel: channelId)
        channelMembership = AmityChannelMembership(client: AmityManager.shared.client!, andChannel: channelId)
    }
    
    func fetchListOfMembers() {
        memberCollection = channelMembership.getMembers(filter: filter, sortBy: sortBy, roles: rolesToFilter)
        participationToken = memberCollection?.observe({ liveCollection, _, error in
            
            Log.add(info: "Membership List Fetched. Data Status: \(liveCollection.dataStatus.description)")
            for i in 0..<liveCollection.count() {
                guard let user = liveCollection.object(at: i) else { return }
                Log.add(info: "User Id: \(user.userId)")
            }
            
            self.dataSourceObserver?.didUpdateDataSource()
        })
    }
    
    // MARK: TableView Helpers
    
    var memberListCount: Int {
        guard let memberCollection = memberCollection else {
            return 0
        }
        
        return Int(memberCollection.count())
    }
    
    func fetchNextPage() {
        guard let memberCollection = memberCollection, memberCollection.hasNext else {
            return
        }
        memberCollection.nextPage()
    }
    
    func getMember(at indexPath: IndexPath) -> AmityChannelMember? {
        guard let memberCollection = memberCollection else {
            return nil
        }
        
        let member = memberCollection.object(at: indexPath.row)
        return member
    }

    public func filterMembers(by role: String?) {
        if let userRole = role {
            let enteredRoles = userRole.trimmingCharacters(in: .whitespacesAndNewlines)
            let roles = enteredRoles.isEmpty ? [] : enteredRoles.components(separatedBy: ",")
            rolesToFilter = roles
        } else {
            rolesToFilter = []
        }
        fetchListOfMembers()
        
        Log.add(info: "Role to filter: \(rolesToFilter)")
    }
    
    public func addMember(id: String, completion: @escaping (Bool, Error?) -> Void) {
        Task { @MainActor in
            do {
                let _ = try await channelMembership.addMembers([id])
                self.dataSourceObserver?.didUpdateDataSource()
                completion(true, nil)
            } catch {
                self.dataSourceObserver?.didUpdateDataSource()
                completion(false, error)
            }
        }
    }
    
    public func removeMember(id: String, completion: @escaping (Bool, Error?) -> Void) {
        Task { @MainActor in
            do {
                let _ = try await channelMembership.removeMembers([id])
                self.dataSourceObserver?.didUpdateDataSource()
                completion(true, nil)
            } catch {
                self.dataSourceObserver?.didUpdateDataSource()
                completion(false, error)
            }
        }
    }
    
    public func addRole(for indexPath: IndexPath, role: String) {
        guard let member = getMember(at: indexPath) else { return }
        Task { @MainActor in
            do {
                let _ = try await channelModeration.addRole(role, userIds: [member.userId])
            } catch {}
            self.dataSourceObserver?.didUpdateDataSource()
        }
    }
    
    public func removeRole(for indexPath: IndexPath, role: String) {
        guard let member = getMember(at: indexPath) else { return }
        Task { @MainActor in
            do {
                let _ = try await channelModeration.removeRole(role, userIds: [member.userId])
            } catch {}
            self.dataSourceObserver?.didUpdateDataSource()
        }
    }
    
    public func banMember(at indexPath: IndexPath, completion: @escaping (Bool, Error?) -> Void) {
        guard let member = getMember(at: indexPath) else { return }
        Task { @MainActor in
            do {
                let success = try await channelModeration.banMembers([member.userId])
                completion(success, nil)
            } catch {
                completion(false, error)
            }
        }
    }
    
    public func unbanMember(at indexPath: IndexPath, completion: @escaping (Bool, Error?) -> Void) {
        guard let member = getMember(at: indexPath) else { return }
        Task { @MainActor in
            do {
                let success = try await channelModeration.unbanMembers([member.userId])
                completion(success, nil)
            } catch {
                completion(false, error)
            }
        }
    }
    
    func muteMember(at indexPath: IndexPath, completion: @escaping (Bool, Error?) -> Void) {
        guard let member = getMember(at: indexPath) else { return }
        
        Task { @MainActor in
            do {
                let success = try await channelModeration.muteMembers([member.userId], mutePeriod: 60)
                completion(success, nil)
            } catch {
                completion(false, error)
            }
        }
    }
    
    func unMuteMember(at indexPath: IndexPath, completion: @escaping (Bool, Error?) -> Void) {
        guard let member = getMember(at: indexPath) else { return }
        
        Task { @MainActor in
            do {
                let success = try await channelModeration.unmuteMembers([member.userId])
                completion(success, nil)
            } catch {
                completion(false, error)
            }
        }
    }
    
    public func updateMembershipFilter(filter: AmityChannelMembershipFilter) {
        self.filter = filter
        self.fetchListOfMembers()
    }
    
    public func updateMembershipSort(sortBy: AmitySortBy) {
        self.sortBy = sortBy
        self.fetchListOfMembers()
    }
}

extension AmityChannelMembershipFilter: CaseIterable {
    public static var allCases: [AmityChannelMembershipFilter] = [.all, .ban, .mute]
    
    var description: String {
        switch self {
        case .all:
            return "All"
        case .ban:
            return "Ban"
        case .mute:
            return "Mute"
        @unknown default:
            fatalError()
        }
    }
    
}
