//
//  ImageChooserView.swift
//  SampleApp
//
//  Created by Nishan Niraula on 15/2/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import SwiftUI
import AmitySDK

struct ImageChooserView: View {
    
    @Environment(\.presentationMode) var presentationMode
    
    @State private var showPhotoPicker: Bool = false
    @State private var showCamera: Bool = false
    @State private var isUploadingImage: Bool = false
    
    @State private var selectedImages: [UIImage] = []
    @State private var capturedImage: UIImage?
    
    // Array of already selected file id
    let existingSelection: [String]
    
    @State private var uploadedFiles: [String] = []
    @State private var showingAlert = false
    
    @State private var errorFromUpload = ""

    // Array of current selection
    let onSelection: (([String]) -> Void)?
    
    @StateObject var imageUploader = AmityImageUploader()
    
    var columns = [GridItem(.adaptive(minimum: 80), spacing: 8, alignment: .center)]
    
    var body: some View {
        ScrollView {
            VStack(alignment: .leading) {
                
                HStack {
                    Text("Add Images")
                        .font(.headline)
                    
                    Spacer()
                    
                    Button {
                        showPhotoPicker = true
                    } label: {
                        Image(systemName: "photo")
                    }
                    .padding(8)
                    .background(Color(uiColor: UIColor.systemGroupedBackground))
                    .cornerRadius(8)
                    
                    Button {
                        showCamera = true
                    } label: {
                        Image(systemName: "camera.fill")
                    }
                    .padding(8)
                    .background(Color(uiColor: UIColor.systemGroupedBackground))
                    .cornerRadius(8)
                    .padding(.leading, 8)
                }
                .padding(.top, 16)
                
                LazyVGrid(columns: columns) {
                    ForEach(selectedImages, id: \.self) { item in
                        Image(uiImage: item)
                            .resizable()
                            .scaledToFit()
                            .frame(width: 80, height: 80)
                    }
                }
                
                HStack {
                    Spacer()
                    
                    Button {
                        isUploadingImage = true
                        imageUploader.uploadImages(images: selectedImages) { uploadedData, uploadedErrors  in
                            self.isUploadingImage = false
                            self.uploadedFiles.append(contentsOf: uploadedData.map { $0.fileId })
                            
                            // Notify about uploaded files
                            onSelection?(uploadedFiles)
                            
                            // Dismiss this screen
                            if uploadedErrors.count == 0 {
                                presentationMode.wrappedValue.dismiss()
                            } else {
                                errorFromUpload = "Upload failed \(uploadedErrors.count) image(s)\n" + uploadedErrors.map({ $0.localizedDescription }).joined(separator: "\n")
                                showingAlert = true
                            }
                        }
                        
                    } label: {
                        Text(isUploadingImage ? "Uploading...." : "Upload Images")
                    }
                    .padding(8)
                    .buttonStyle(.bordered)
                    .alert(isPresented: $showingAlert) {
                        Alert(title: Text("Error"), message: Text(errorFromUpload), dismissButton: .cancel(Text("Okay"), action: {
                            presentationMode.wrappedValue.dismiss()
                        }))
                    }
                    
                    Spacer()
                }
                .padding(.top, 32)
            }
            .padding(.horizontal, 16)
        }
        .sheet(isPresented: $showPhotoPicker) {
            PhotoPicker(pickerResult: $selectedImages, isPresented: $showCamera)
        }
        .sheet(isPresented: $showCamera) {
            CameraPicker(image: $capturedImage)
        }
        .onChange(of: capturedImage) { newValue in
            guard let newValue else { return }
            
            selectedImages.append(newValue)
        }
        .task {
            uploadedFiles = existingSelection
        }
    }
}

struct ImageChooserView_Previews: PreviewProvider {
    static var previews: some View {
        ImageChooserView(existingSelection: [], onSelection: nil)
    }
}

class AmityImageUploader: ObservableObject {

    struct Uploadable {
        let id: Int
        let image: UIImage
    }
    
    struct UploadedData {
        let id: Int
        let data: AmityImageData
    }
    
    let fileRepo = AmityFileRepository(client: AmityManager.shared.client!)
    
    func uploadImages(images: [UIImage], completion: @escaping ([AmityImageData], [Error]) -> Void) {
        
        Task { @MainActor in
            var uploadedInfo = [UploadedData]()
            var uploadedErrors = [Error]()
            
            for (index, image) in images.enumerated() {
                // Prepare for upload
                let itemToUpload = Uploadable(id: index, image: image)
                do {
                    let result = try await fileRepo.uploadImage(itemToUpload.image, progress: nil)
                    let uploadedData = UploadedData(id: itemToUpload.id, data: result)
                    uploadedInfo.append(uploadedData)
                } catch {
                    uploadedErrors.append(error)
                }
            }
            completion(uploadedInfo.map { $0.data }, uploadedErrors)
        }
    }
}
