//
//  NewUserPostsViewController.swift
//  SampleApp
//
//  Created by Nishan Niraula on 4/27/20.
//  Copyright © 2020 David Zhang. All rights reserved.
//

import UIKit
import AmitySDK
import MobileCoreServices
import AVFoundation

protocol NewUserPostsViewControllerDelegate: AnyObject {
    func newUserPostsViewControllerDidCreateNewPost(_ controller: NewUserPostsViewController)
    func newUserPostsViewControllerDidUpdateComment()
}

/*
 * Note:
 *
 * This class displays the screen to edit/create new post.
 */
class NewUserPostsViewController: UIViewController, UINavigationControllerDelegate {
    
    weak var delegate: NewUserPostsViewControllerDelegate?
    
    @IBOutlet weak var scrollView: UIScrollView!
    
    @IBOutlet weak var textView: UITextView!
    @IBOutlet weak var attachImageButton: UIButton!
    @IBOutlet weak var attachVideoButton: UIButton!
    @IBOutlet weak var attachFileButton: UIButton!
    @IBOutlet weak var imageCountLabel: UILabel!
    @IBOutlet weak var videoCountLabel: UILabel!
    @IBOutlet weak var fileCountLabel: UILabel!
    
    @IBOutlet private weak var addMediasStackView: UIStackView!
    
    // Update Medias IBOutlets
    @IBOutlet private weak var updateMediasStackView: UIStackView!
    @IBOutlet private weak var updateMediasButton: UIButton!
    @IBOutlet private weak var mediasChangesCommitedLabel: UILabel!
    
    // Poll IBOutlets
    @IBOutlet weak var pollPostSwitch: UISwitch!
    @IBOutlet weak var numberOfPollTextField: UITextField!
    @IBOutlet weak var numberOfDayToClosePollTextField: UITextField!
    @IBOutlet weak var pollMultipleVodeSwitch: UISwitch!
    @IBOutlet weak var pollView: UIView!
    
    @IBOutlet weak var communityPostSwitch: UISwitch!
    @IBOutlet weak var communityIdField: UITextField!
    @IBOutlet weak var streamIdTextField: UITextField!
    
    // Members tableview for mention
    @IBOutlet private weak var membersTableView: UITableView!
    @IBOutlet private weak var tableViewHeightConstraint: NSLayoutConstraint!
    
    private var videosData: [AmityVideoData]?
    private var imagesData: [AmityImageData]?
    private var filesData: [AmityFileData]?
    private var isMediaEmpty: Bool {
        return videosData?.isEmpty ?? true
        && imagesData?.isEmpty ?? true
        && filesData?.isEmpty ?? true
    }
    
    var feedManager: UserPostsFeedManager!
    var commentManager: UserPostCommentManager!
    var isPostEnabled = true
    
    private var mentionManager: AmityMentionManager?
    
    var communityId: String = ""
    var isCommunityPost: Bool = false
    
    enum ModeType {
        case post
        case comment
    }
    
    enum Mode {
        case create(type: ModeType = .post)
        case edit(type: ModeType = .post)
    }
    
    var currentMode: Mode = .create(type: .post)
    
    override func viewDidLoad() {
        super.viewDidLoad()
        setupViews()
        
        if let manager = feedManager, let community = manager.community {
            communityIdField.text = community.communityId
        }
        
        if let manager = commentManager {
            communityIdField.text = manager.communityId
        }
        
        setupMentionManager()
        textView.autocorrectionType = .no
    }
    
    func setupViews() {
        
        scrollView.contentInset.bottom = 350
        textView.keyboardDismissMode = .onDrag
        textView.font = UIFont.systemFont(ofSize: 16, weight: .regular)
        textView.delegate = self
        attachImageButton.addTarget(self, action: #selector(onAttachImageButtonTap), for: .touchUpInside)
        attachVideoButton.addTarget(self, action: #selector(onAttachVideoButtonTap), for: .touchUpInside)
        
        attachVideoButton.addTarget(self, action: #selector(onAttachVideoButtonTap), for: .touchUpInside)
        attachFileButton.addTarget(self, action: #selector(onAttachFileButtonTap), for: .touchUpInside)
        updateViewsForCurrentMode()
        pollView.isHidden = true
        
        if isCommunityPost {
            communityPostSwitch.isOn = true
            communityIdField.text = communityId
        }

        let nib = UINib(nibName: "SampleAppTableViewCell", bundle: nil)
        membersTableView?.register(nib, forCellReuseIdentifier: "SampleAppTableViewCell")
        
        membersTableView?.tableFooterView = UIView()
        
        membersTableView.dataSource = self
        membersTableView.delegate = self
        
        hideMentionsTableView()
        updateMediaButtonState()
    }
    
    private func updateMediaButtonState(dataChanges: Bool = false) {
        // Medias update UI
        
        switch currentMode {
        case .edit(let type):
            if type == .post {
                // post doesn't support deleting one media type and adding another one in one call.
                // to prevent the situation, we disable media button once data changes.
                //
                // in order to change attachment type, we can instead do following steps.
                // 1. remove all attachments and click update
                // 2. edit post, add new attachment type and click update
                if dataChanges {
                    addMediasStackView.isHidden = true
                    updateMediasStackView.isHidden = false
                    updateMediasButton.isEnabled = false
                } else {
                    addMediasStackView.isHidden = !isMediaEmpty
                    updateMediasStackView.isHidden = isMediaEmpty
                    updateMediasButton.isEnabled = true
                }
                let currentText = feedManager.editedPost?.data?["text"] as? String
                self.textView.text = currentText
            } else {
                addMediasStackView.isHidden = true
                updateMediasStackView.isHidden = true
                let currentText = commentManager.getEditCommentData()?.text
                self.textView.text = currentText
            }
        case .create(type: let type):
            if type == .post {
                addMediasStackView.isHidden = !isMediaEmpty
                updateMediasStackView.isHidden = isMediaEmpty
            } else {
                addMediasStackView.isHidden = true
                updateMediasStackView.isHidden = true
            }
        }
        mediasChangesCommitedLabel.isHidden = true
    }
    
    private func setupMentionManager() {
        var managerType: AmityMentionManagerType

        switch currentMode {
        case .create(let type), .edit(let type):
            if type == .comment {
                managerType = .comment(communityId: commentManager?.communityId)
            } else {
                managerType = .post(communityId: (feedManager?.community?.isPublic ?? true) ? nil : feedManager?.community?.communityId ?? "")
            }
        }

        mentionManager = AmityMentionManager(withType: managerType)
        mentionManager?.delegate = self

        switch currentMode {
        case .create:
            break
        case .edit(let type):
            var metadata: [String: Any]?
            var text = ""

            if type == .comment {
                let comment = commentManager?.getEditCommentData()
                text = comment?.text ?? ""
                metadata = comment?.metadata
            } else {
                let post = feedManager?.getEditPostData()
                text = post?.text ?? ""
                metadata = post?.metadata
                
                if let childrenPosts = feedManager.editedPost?.childrenPosts,
                   let dataType = childrenPosts.first?.dataType {
                    switch dataType {
                    case "video":
                        videosData = childrenPosts.compactMap { post in post.getVideoInfo() }
                    case "image":
                        imagesData = childrenPosts.compactMap({ post in post.getImageInfo() })
                    case "file":
                        filesData = childrenPosts.compactMap({ post in post.getFileInfo() })
                    default:
                        break
                    }
                    updateMediaButtonState()
                }
            }

            if let metadata = metadata {
                mentionManager?.setMentions(metadata: metadata, inText: text)
            }
        }
    }
    
    func updateViewsForCurrentMode() {
        
        let postButton: UIBarButtonItem
        let cancelButton: UIBarButtonItem = UIBarButtonItem(title: "Cancel", style: .plain, target: self, action: #selector(onCancelButtonTap))
        
        
        switch currentMode {
        case .create(let type):
            self.title = "Create \(type == .comment ? "Comment" : "Post")"
            
            postButton = UIBarButtonItem(title: "Post", style: .plain, target: self, action: #selector(onPostButtonTap))
        case .edit(let type):
            self.title = "Edit \(type == .comment ? "Comment" : "Post")"
            
            postButton = UIBarButtonItem(title: "Update", style: .plain, target: self, action: #selector(onPostButtonTap))
        }
        
        self.navigationItem.rightBarButtonItem = postButton
        self.navigationItem.leftBarButtonItem = cancelButton
    }
    
    @objc func onPostButtonTap() {
        
        guard isPostEnabled else { return }
        
        let postText = textView.text
        
        isPostEnabled = false
        
        let metadata = mentionManager?.getMetadata()
        let mentionees = mentionManager?.getMentionees()
        
        switch currentMode {
        case .create:
            
            let communityPostId = communityIdField.text ?? ""
            let isCommunityPost = communityPostSwitch.isOn && !communityPostId.isEmpty
            
            if pollPostSwitch.isOn {
                feedManager.createPollPost(text: postText, numOptions: numberOfPollTextField.text, numDayToClose: numberOfDayToClosePollTextField.text, isMultipleVote: pollMultipleVodeSwitch.isOn, communityId: isCommunityPost ? communityPostId : nil, metadata: metadata, mentionees: mentionees) { [weak self] result in
                    guard let strongSelf = self else { return }
                    strongSelf.isPostEnabled = true
                    strongSelf.showAlertAndDismiss(result: result)
                    strongSelf.delegate?.newUserPostsViewControllerDidCreateNewPost(strongSelf)
                }
            } else {
                feedManager.createPost(text: postText, images: imagesData, videos: videosData, files: filesData, communityId: isCommunityPost ? communityPostId : nil, streamId: streamIdTextField.text, metadata: metadata, mentionees: metadata == nil ? nil : mentionees) { [weak self] result in
                    guard let strongSelf = self else { return }
                    strongSelf.isPostEnabled = true
                    strongSelf.showAlertAndDismiss(result: result)
                    strongSelf.delegate?.newUserPostsViewControllerDidCreateNewPost(strongSelf)
                }
            }
            
        case let .edit(type):
            switch type {
            case .post:
                feedManager.updatePost(text: postText, images: imagesData, videos: videosData, files: filesData, metadata: metadata, mentionees: mentionees) { [weak self] (result) in
                    guard let strongSelf = self else {
                        return
                    }
                    strongSelf.isPostEnabled = true
                    strongSelf.showAlertAndDismiss(result: result)
                    strongSelf.delegate?.newUserPostsViewControllerDidCreateNewPost(strongSelf)
                }
            case .comment:
                break
            }
        }
        
    }
    
    func showAlertAndDismiss(result: Result<AmityPost, Error>) {
        var isEditMode = false
        if case .edit = currentMode {
            isEditMode = true
        }
        
        var alertTitle = ""
        var message = ""
        switch result {
        case .success:
            alertTitle = "Post successfully \(isEditMode ? "updated" : "created")"
        case .failure(let error):
            alertTitle = "Error while \(isEditMode ? "updating" : "creating") post"
            message = error.localizedDescription
        }
        
        let alert = UIAlertController(title: alertTitle, message: message, preferredStyle: .alert)
        let action = UIAlertAction(title: "Dismiss", style: .cancel, handler: { action in
            self.dismiss(animated: true, completion: nil)
        })
        alert.addAction(action)
        
        self.present(alert, animated: true, completion: nil)
    }
    
    func showAlertAndDismiss(message: String) {
        let alert = UIAlertController(title: nil, message: message, preferredStyle: .alert)
        let action = UIAlertAction(title: "Dismiss", style: .cancel, handler: { action in
            self.dismiss(animated: true, completion: nil)
        })
        alert.addAction(action)
        
        self.present(alert, animated: true, completion: nil)
    }
    
    
    // MARK: - Create Poll
    @IBAction func onTapPollSwitch(_ sender: UISwitch) {
        pollView.isHidden = !sender.isOn
    }
    
    @objc func onCancelButtonTap() {
        self.dismiss(animated: true, completion: nil)
    }
    
    @objc func onAttachImageButtonTap() {
        presentMediasPicker(mediaType: .image)
    }
    
    @objc func onAttachVideoButtonTap() {
        presentMediasPicker(mediaType: .video)
    }
    
    @objc func onAttachFileButtonTap() {
        presentMediasPicker(mediaType: .file)
    }
    
    @IBAction func updateMediasButtonDidTouch() {
        
        let mediaListController: MediaListController?
        
        if let videosData = videosData, !videosData.isEmpty {
            mediaListController = MediaListController(mediaType: .video)
            mediaListController?.videos = videosData
        } else if let imagesData = imagesData, !imagesData.isEmpty {
            mediaListController = MediaListController(mediaType: .image)
            mediaListController?.images = imagesData
        } else if let filesData = filesData, !filesData.isEmpty {
            mediaListController = MediaListController(mediaType: .file)
            mediaListController?.files = filesData
        } else {
            mediaListController = nil
        }
        guard let mediaListController = mediaListController else {
            let alertController = UIAlertController(
                title: "Unsupported",
                message: "Unable to update media list for post with videos(\(videosData?.count ?? 0)) images(\(imagesData?.count ?? 0)) files(\(filesData?.count ?? 0))",
                preferredStyle: .alert
            )
            alertController.addAction(UIAlertAction(title: "OK", style: .cancel))
            present(alertController, animated: true, completion: nil)
            return
        }
        mediaListController.delegate = self
        navigationController?.pushViewController(mediaListController, animated: true)
    }
    
    private func presentMediasPicker(mediaType: MediaListController.MediaType) {
        let mediaListController = MediaListController(mediaType: mediaType)
        
        mediaListController.delegate = self
        navigationController?.pushViewController(mediaListController, animated: true)
    }
    
    private func hideMentionsTableView() {
        tableViewHeightConstraint.constant = 0
        membersTableView?.isHidden = true
    }
    
}

extension NewUserPostsViewController: UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        mentionManager?.addMention(from: textView, in: textView.text, at: indexPath)
    }
}

extension NewUserPostsViewController: UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return mentionManager?.itemsCount ?? 0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        guard let cell = tableView.dequeueReusableCell(withIdentifier: "SampleAppTableViewCell") as? SampleAppTableViewCell else { return UITableViewCell() }
        
        if let model = mentionManager?.item(at: indexPath) {
            cell.titleLabel?.text = model.displayName
        }
        
        return cell
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 40.0
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        if indexPath.row == (mentionManager?.itemsCount ?? 0) - 4 {
            mentionManager?.loadMore()
        }
    }
}

extension NewUserPostsViewController: AmityMentionManagerDelegate {
    func didGetUsers(users: [AmityMentionUserModel]) {
        if users.isEmpty {
            tableViewHeightConstraint.constant = 0
            membersTableView.isHidden = true
        } else {
            var heightConstraint: CGFloat = 240.0
            if users.count < 5 {
                heightConstraint = CGFloat(users.count) * 52.0
            }
            tableViewHeightConstraint.constant = heightConstraint
            membersTableView.isHidden = false
            membersTableView.reloadData()
        }
    }
    
    func didCreateAttributedString(attributedString: NSAttributedString) {
        textView.attributedText = attributedString
    }
    
    func didMentionsReachToMaximumLimit() {
        let message = "Mentions are reached to maximum limit"
        let alertController = UIAlertController(title: "Unable to mention", message: message, preferredStyle: .alert)
        let cancelAction = UIAlertAction(title: "Done", style: .cancel, handler: nil)
        alertController.addAction(cancelAction)
        present(alertController, animated: true, completion: nil)
    }
    
    func didCharactersReachToMaximumLimit() {
        showAlertForMaximumCharacters()
    }
    
    private func showAlertForMaximumCharacters() {
        let title = "Unable to mention"
        let message = "Text reached to maximum characters limit"
        let alertController = UIAlertController(title: title, message: message, preferredStyle: .alert)
        let cancelAction = UIAlertAction(title: "Done", style: .cancel, handler: nil)
        alertController.addAction(cancelAction)
        present(alertController, animated: true, completion: nil)
    }
}

extension NewUserPostsViewController: UITextViewDelegate {
    func textView(_ textView: UITextView, shouldChangeTextIn range: NSRange, replacementText text: String) -> Bool {
        if textView.text.count > AmityMentionManager.maximumCharacterCountForPost {
            showAlertForMaximumCharacters()
            return false
        }
        return mentionManager?.shouldChangeTextIn(textView, inRange: range, replacementText: text, currentText: textView.text) ?? true
    }
    
    func textViewDidChangeSelection(_ textView: UITextView) {
        mentionManager?.changeSelection(textView)
    }
}

extension NewUserPostsViewController: MediaListControllerDelegate {
    
    func mediaListControllerDidSaveMediaList(_ controller: MediaListController) {
        switch controller.mediaType {
        case .video:
            videosData = controller.videos
            imagesData = nil
            filesData = nil
        case .image:
            imagesData = controller.images
            videosData = nil
            filesData = nil
        case .file:
            filesData = controller.files
            imagesData = nil
            videosData = nil
        }
        
        updateMediaButtonState(dataChanges: true)
        mediasChangesCommitedLabel.isHidden = false
        
        imageCountLabel.text = "\(imagesData?.count ?? 0) Image attached"
        videoCountLabel.text = "\(videosData?.count ?? 0) Video attached"
        fileCountLabel.text = "\(filesData?.count ?? 0) File attached"
    }
    
}
