//
//  StoryCollectionView.swift
//  SampleApp
//
//  Created by Manuchet Rungraksa on 4/12/2566 BE.
//  Copyright © 2566 BE David Zhang. All rights reserved.
//

import Foundation
import SwiftUI
import AmitySDK

struct StoryCollectionView: View {
    
    @ObservedObject var storyCollection: AmityCollection<AmityStory>
    @ObservedObject var storyTargetObject: AmityObject<AmityStoryTarget>
    let viewModel: StoryListViewModel
    
    @Environment(\.navigationController) var navigationController
    @State private var selection: String?
    @State private var showAlert = false
    @State private var alertText = ""
    @State var reachUserListView: ReachUserListView?

    var body: some View {
        Text("Has unseen story: \(storyTargetObject.snapshot?.hasUnseen ?? false ? "Yes" :  "No")")
        List(storyCollection.snapshots, id: \.storyId) { item in
            let story = StoryModel(story: item)
            storyView(story)
        }
        .listStyle(.plain)
    }
    
    func storyView(_ story: StoryModel) -> some View {
        ZStack {
            
            StoryItemView(model: story, hardDeleteAction: {
                deleteStory(storyId: story.id, hardDelete: true)
            }, softDeleteAction: {
                deleteStory(storyId: story.id, hardDelete: false)
            }, markAsSeen: {
                viewModel.markSeen(story: story.storyObject)
                alertText = "Successfully mark story as seen"
                showAlert = true
            }, getSeenUsers: {
                let viewmodel = ReachUserListViewModel(viewedId: story.storyId, viewedType: .story, impressionCount: story.impression, reachCount: story.reach)
                reachUserListView = ReachUserListView(viewModel: viewmodel)

                self.selection = story.storyId + "+story"

            }, markLinkAsClicked: {
                viewModel.markLinkAsClicked(story: story.storyObject)
                alertText = "Successfully mark link as clicked"
                showAlert = true
            })
            .alert(isPresented: $showAlert) {
                Alert(
                    title: Text(alertText)
                )
                
            }
            NavigationLink(
                destination: reachUserListView,
                tag: story.storyId + "+story",
                selection: self.$selection,
                label: {
                    EmptyView()
                })
            .buttonStyle(PlainButtonStyle())
            .hidden()
        }
    }
    
    func deleteStory(storyId: String, hardDelete: Bool) {
        Task {
            do {
                try await viewModel.deleteStory(storyId: storyId, hardDelete: hardDelete)
                
                alertText = "Story successfully Deleted"
                showAlert = true
                
            } catch {
                alertText = error.localizedDescription
                showAlert = true
            }
        }
    }
}

// Concrete implementor
class StoryListViewModel: ObservableObject {
    
    var pageTitle: String
    var targetId: String
    
    private let storyRepository = AmityStoryRepository(client: AmityManager.shared.client!)
    
    @Published var storyCollection: AmityCollection<AmityStory>
    @Published var storyTargetObject: AmityObject<AmityStoryTarget>
        
    var storyManager = StoryManager()
    
    var currentSortOption: AmityStoryQuerySortOption = .lastCreated
    
    @MainActor
    init(targetId: String) {
        self.pageTitle = "Story in \(targetId)"
        self.targetId = targetId
        storyCollection = storyManager.getActiveStories(communityId: targetId, sortOptions: currentSortOption)
        storyTargetObject = storyManager.getStoryTarget(communityId: targetId)
    }
    
    @MainActor
    func queryStory() {
        storyCollection = storyManager.getActiveStories(communityId: targetId, sortOptions: currentSortOption)
        
    }
    
    @MainActor 
    func queryStoryTarget() {
        storyTargetObject = storyManager.getStoryTarget(communityId: targetId)
        
    }
    
    func setSort(_ sort: AmityStoryQuerySortOption) {
        self.currentSortOption = sort
    }
        
    func deleteStory(storyId: String, hardDelete: Bool) async throws {
        try await storyManager.deleteStory(storyId: storyId, hardDelete: hardDelete)
    }
    
    func markSeen(story: AmityStory) {
        story.analytics.markAsSeen()
    }
    
    func markLinkAsClicked(story: AmityStory) {
        story.analytics.markLinkAsClicked()
    }
    
}
